@@ -6,14 +6,12 @@
 
 # in __init__.py
 from sysmpy.entity import *
+from sysmpy.entity_db import *
 from sysmpy.relationship import *
-from sysmpy.interfaceanalyzer import *
+from sysmpy.interface_analyzer import *
 from sysmpy.matrix_for_graph import *
 from sysmpy.script_to_model import *
 from sysmpy.script_generator import *
+from sysmpy.model_generator import *
 from sysmpy.sms_searcher import *
 from sysmpy.gui.gui_main import *
-
-# Only these modules will be imported with "import *"
-# __all__ = ["entity", "relationship"]
-
