@@ -4,6 +4,9 @@ import random
 import queue
 import traceback
 import os
+from copy import deepcopy
+import sysmpy.entity_db as entity_db
+from sysmpy.util import *
 
 
 class Entity():
@@ -11,13 +14,9 @@ class Entity():
     An entity is something can exist by itself and is uniquely identifiable.
     """
 
-    # This is a database for all entities
-    entity_db = []
     _debug_mode = False
 
     def __init__(self, name, number=None, description=None):
-        super().__init__()
-
         self.name = name
         self.number = number
         self.description = description
@@ -26,9 +25,11 @@ class Entity():
         self.sim_with_decomposition = True
         self.is_root = False
         self.is_decomposed = False
+        self.module = None
 
         # All entities created are stored in the static class Entity
-        Entity.store_entity(self)
+        entity_db.store_entity(self)
+        # Entity.store_entity(self)
 
     def __str__(self):
         return f"{self.name} is associated with {self.relation}"
@@ -210,93 +211,15 @@ class Entity():
         self.find_all_nodes(Loop, ret)
         return ret[0].start.end
 
-    ########################################################################
-    # Static Methods
-    # Get or Search Methods
-    # The following methods is for a global entity
-    #
-
+    # ########################################################################
+    # # Static Methods
+    # # Get or Search Methods
+    # # The following methods is for a global entity
+    # #
     @staticmethod
     def debug_mode(b):
         Entity._debug_mode = b
 
-    @staticmethod
-    def store_entity(self):
-        """ All entities created are stored """
-
-        Entity.entity_db.append(self)
-
-    @staticmethod
-    def clear_entity(self):
-        """ All entities created are stored """
-        Entity.entity_db.clear()
-
-    @staticmethod
-    def get(name_entity):
-        """ This returns instances of an entity """
-
-        # Find a module which is using this get function:
-        # - last element ([-1]) is me, the one before ([-2]) is my caller.
-        # - The first element in caller's data is the filename
-        caller_path = traceback.extract_stack()[-2][0]
-        caller, file_extension = os.path.splitext(caller_path)
-
-        l = name_entity.split('.')
-
-        cur_obj = None
-        for i in l:
-            if cur_obj is None:
-                cur_obj = Entity.get_op(i, caller)
-            else:
-                cur_obj = Entity.get_by_relationship(cur_obj, i, 'contains')
-
-        return cur_obj
-
-    @staticmethod
-    def get_by_relationship(obj_parent, name_target, relationship):
-        """
-        This returns instances of an entity in a parent entity by using the relationship name
-        e.g.,)
-        cur_obj = Component()
-        Entity.get_by_relationship(cur_obj, 'size', 'contains')
-
-        """
-
-        if relationship in obj_parent.relation:
-            relations = [x.end for x in obj_parent.relation[relationship]]
-            if relations is not None:
-                e = [x for x in relations if x.name == name_target]
-                if e is None:
-                    return None
-                elif len(e) == 1:
-                    return e[0]
-                else:
-                    return e
-
-    @staticmethod
-    def get_op(name_entity, caller=None):
-        """ This returns instances of an entity by searching in the entity_db"""
-        if caller is None:
-            e = [x for x in Entity.entity_db if x.name == name_entity]
-        elif caller is not None:
-            e = [x for x in Entity.entity_db if x.name == name_entity and x.module == caller]
-
-        if e is None:
-            return None
-        elif len(e) == 1:
-            return e[0]
-        else:
-            return e
-
-    @staticmethod
-    def get_by_type(type_entity):
-        """ This returns instances of an entity by the entity type"""
-        e = [x for x in Entity.entity_db if isinstance(x, type_entity)]
-        if e is None:
-            return None
-        else:
-            return e
-
 ########################################################################
 # Static Entity
 #
@@ -341,8 +264,8 @@ class Property(StaticEntity):
         ['Between(1, 10)', 'Between(30, 40)']
         :param value: e.g., ) 'T', 10
         """
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         self.range = range
         self.value = value
 
@@ -354,8 +277,8 @@ class Property(StaticEntity):
 
 class Item(StaticEntity):
     def __init__(self, name):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         self.attr = {}
 
         # Item size is used for conduit operation
@@ -372,8 +295,8 @@ class Conduit(StaticEntity):
     """
 
     def __init__(self, name):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         self._capacity = 1
         self._delay = 0
 
@@ -405,8 +328,8 @@ class Conduit(StaticEntity):
 
 class Resource(StaticEntity):
     def __init__(self, name, amount=10, minimum=1, maximum=10, units=None):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         self.amount = amount
         self.minimum = minimum
         self.maximum = maximum
@@ -415,8 +338,8 @@ class Resource(StaticEntity):
 
 class Requirement(Property):
     def __init__(self, name, range=None, value=None, **kwargs):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name, range, value)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         self.kwargs = kwargs
         self.is_root = True
 
@@ -456,8 +379,8 @@ class Requirement(Property):
 
 class Component(StaticEntity):
     def __init__(self, name, **kwargs):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         self.kwargs = kwargs
         self.is_root = True
 
@@ -497,9 +420,20 @@ class DynamicEntity(Entity):
         self.function = None
         self.sim_network = []
 
-    def reset(self):
+    def reset_waiting(self):
         self.waiting = False
 
+    def reset_sim_status(self):
+        self.waiting = False
+        self.time = 0
+        self.total_time = 0
+        self.is_root = False
+
+        for rels in self.relation:
+            rel = self.relation[rels]
+            for r in rel:
+                r.reset_sim_status()
+
     ########################################################################
     # Relationship Methods
     # !start with an lowercase letter
@@ -587,19 +521,49 @@ class DynamicEntity(Entity):
     # Simulation Methods
     #
     #
+    def make_network(self):
+        """
+        This create a simulation network containing flows between nodes
+        :return:
+        """
+
+        # Make a new entity corresponding to this entity
+        # After using the new entity, it will be removed.
+        new_en = deepcopy(self)
+
+        new_en.make_network_op(new_en.sim_network)
+
+        return new_en
+
     def init_sim_network(self):
         """
         This create a simulation network containing flows between nodes
         :return:
         """
-        if len(self.sim_network) == 0:
-            self.init_sim_network_op(self.sim_network)
+        # Make a network for the action model
+        new_en = self.make_network()
 
-        for en in self.sim_network:
-            en.time = 0
-            en.total_time = 0
+        # Reset all states in the elements of the network
+        for en in new_en.sim_network:
+            en.reset_sim_status()
 
-    def init_sim_network_op(self, entities):
+        new_en.reset_sim_status()
+
+        # The function sim is called, which means this is a root process
+        new_en.is_root = True
+        new_en.end.is_root = True
+
+        # The root process is linked to 'process_END' using the relation 'flow'
+        # 'process_END' -> 'process'
+        new_en.end.flow(new_en)
+
+        # For the relation 'process_END' -> 'process', set 'sent = true' to activate the root process
+        flowed_from = [r for r in new_en.inv_relation['flowed from'] if isinstance(r, Flow)]
+        flowed_from[0].sent = True
+
+        return new_en
+
+    def make_network_op(self, entities):
         """
         This finds all node flows
         The returning list acts will have flow nodes which are activated by Python Asyncio
@@ -616,7 +580,7 @@ class DynamicEntity(Entity):
                 last_e = []
                 for e in contains:
 
-                    if isinstance(self, And) or isinstance(self, Or) or isinstance(self, Condition):
+                    if isinstance(self, And) or isinstance(self, Or) or isinstance(self, XOr) or isinstance(self, Condition):
                         # If e is not 'or' or 'and', then make a parallel flow
                         self.flow(e)
                         last_e.append(e)
@@ -642,7 +606,7 @@ class DynamicEntity(Entity):
 
                     # If this entity does not allow the decomposition, don't make any sub-flows for simulation
                     if e.sim_with_decomposition:
-                        e.init_sim_network_op(entities)
+                        e.make_network_op(entities)
 
                     # If this entity is Action and the current child is Process,
                     # this means this is a decomposed entity
@@ -706,7 +670,6 @@ class DynamicEntity(Entity):
         while Process.activated:
             # Info is used to store any information in this process and be printed out by 'Process.store_event()'
             info = ''
-
             ##################################################################################
             #
             #       Phase 1: Check Activating Conditions
@@ -882,6 +845,7 @@ class DynamicEntity(Entity):
             #       Phase 2: Perform Action
             #
             ##################################################################################
+
             if b_flow_fire and b_seizes_fire and b_consumes_fire and b_triggered_fire:
 
                 ########################################################################
@@ -963,17 +927,26 @@ class DynamicEntity(Entity):
                     if isinstance(self, Action):
                         if self.function is not None:
                             # Perform a function script for selection
-                            selected = self.function(Entity)
+                            selected = self.function(entity_db)
 
-                    if isinstance(self, Or):
+                    if isinstance(self, XOr): # Select only one process
                         selected = random.choice(flows)
                         Process.store_event(self, 'selects ' + selected.end.name)
                         selected.sent = True
+                    if isinstance(self, Or): # Select at least one process
+                        or_flows = or_selector(flows)
+                        selected = random.choice(or_flows)
+                        for sel in selected:
+                            if sel is not None:
+                                Process.store_event(self, 'selects ' + sel.end.name)
+                                sel.sent = True
                     elif isinstance(self, Condition):
                         if self.function is not None:
                             # Perform a function script for selection
-                            selected = self.function(Entity)
-                            selected_flows = [r for r in flows if r.end is selected]
+                            selected = self.function(entity_db)
+                            # Because 'selected' is the original process in db and r.end is a copied process,
+                            # entity_db.is_same() is used to check their same identity.
+                            selected_flows = [r for r in flows if entity_db.is_same(r.end, selected)]
                             Process.store_event(self, 'selects ' + ', '.join([r.end.name for r in selected_flows]))
                             for r in selected_flows:
                                 r.sent = True
@@ -987,7 +960,7 @@ class DynamicEntity(Entity):
 
                         if self.count >= self.times-1 or len(exit_loop) > 0:
                             Process.store_event(self, 'finished repetition')
-                            self.reset()
+                            self.reset_waiting()
                             loops = [x for x in self.relation['flows'] if not isinstance(x.end, Loop)]
                             for x in loops:
                                 x.sent = True
@@ -999,10 +972,11 @@ class DynamicEntity(Entity):
                                 x.sent = True
 
                     elif isinstance(self, Process_END) and self.is_root:
-                        self.reset()
+                        self.reset_waiting()
                         for x in flows:
                             x.sent = True
                     else:
+                        # 'And' or other entities will send all flows to their child entities.
                         for x in flows:
                             x.sent = True
 
@@ -1035,7 +1009,6 @@ class DynamicEntity(Entity):
 
                     # Save proper simulation results
                     proc = self.get_pairs_inv()
-                    proc.save_simulation_results()
 
                     if Process.global_max is None:
                         Process.store_event(self, 'deactivating root process now', info='')
@@ -1052,8 +1025,8 @@ class Action(DynamicEntity):
     Action Dynamic Entity
     """
     def __init__(self, name, duration=1):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name, duration)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
 
     ########################################################################
     # Class Creation Methods
@@ -1086,8 +1059,8 @@ class And(DynamicEntity):
     And Dynamic Entity
     """
     def __init__(self, name='and'):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
 
         # This decomposes a pair dynamic entity called 'end' or name+'_END'
         self.end = And_END(name + '_END')
@@ -1118,8 +1091,8 @@ class Or(DynamicEntity):
     Or Dynamic Entity
     """
     def __init__(self, name='or'):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
 
         # This decomposes a pair dynamic entity called 'end' or name+'_END'
         self.end = Or_END(name + '_END')
@@ -1145,25 +1118,21 @@ class Or_END(DynamicEntity):
         super().__init__(name)
 
 
-class Loop(DynamicEntity):
+class XOr(DynamicEntity):
     """
-    Loop Dynamic Entity
+    XOr Dynamic Entity
     """
 
-    def __init__(self, name='loop', times=2):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
+    def __init__(self, name='xor'):
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
 
         # This decomposes a pair dynamic entity called 'end' or name+'_END'
-        self.end = Loop_END(name + '_END', times=times)
+        self.end = XOr_END(name + '_END')
 
         # Link this and the end using the relation 'Pairs'
         Pairs(self, self.end)
 
-        # 'loop' is linked to 'loop_END' using the relation 'flow'
-        # 'loop_END' -> 'loop'
-        self.end.flow(self)
-
     ########################################################################
     # Class Creation Methods
     # !start with an uppercase letter
@@ -1177,18 +1146,10 @@ class Loop(DynamicEntity):
         return obj
 
 
-class Loop_END(DynamicEntity):
-    def __init__(self, name, times):
+class XOr_END(DynamicEntity):
+    def __init__(self, name):
         super().__init__(name)
 
-        # 'loop_END' is set with the looping times
-        self.times = times
-        self.count = 0
-
-    def reset(self):
-        super().reset()
-        self.count = 0
-
 
 class Condition(DynamicEntity):
     """
@@ -1196,8 +1157,8 @@ class Condition(DynamicEntity):
     """
 
     def __init__(self, name):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
 
         # This decomposes a pair dynamic entity called 'end' or name+'_END'
         self.end = Condition_END(name + '_END')
@@ -1222,8 +1183,8 @@ class Condition_END(DynamicEntity):
     def __init__(self, name):
         super().__init__(name)
 
-    def reset(self):
-        super().reset()
+    def reset_waiting(self):
+        super().reset_waiting()
 
 
 class END(DynamicEntity):
@@ -1232,19 +1193,19 @@ class END(DynamicEntity):
     """
 
     def __init__(self, name):
-        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
         super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
 
-    def reset(self):
-        super().reset()
+    def reset_waiting(self):
+        super().reset_waiting()
 
 
 class ExitLoop(DynamicEntity):
     def __init__(self, name):
         super().__init__(name)
 
-    def reset(self):
-        super().reset()
+    def reset_waiting(self):
+        super().reset_waiting()
 
 
 class Process(DynamicEntity):
@@ -1264,27 +1225,28 @@ class Process(DynamicEntity):
     event_queue = queue.Queue()
 
     def __init__(self, name):
+        super().__init__(name)
         # Find a module which is using this class
         # and set the caller's path to this class, so we can know where this object came form:
         # - last element ([-1]) is me, the one before ([-2]) is my caller.
         # - The first element in caller's data is the filename
         self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
 
-        super().__init__(name)
+        # Loop can inherit process. And a pure process uses the following codes.
+        if isinstance(self, Loop) is False:
+            self.is_root = True
 
-        self.is_root = True
+            # This decomposes a pair dynamic entity called 'end' or name+'_END'
+            self.end = Process_END(name + '_END')
 
-        # This decomposes a pair dynamic entity called 'end' or name+'_END'
-        self.end = Process_END(name + '_END')
+            # Link this and the end using the relation 'Pairs'
+            Pairs(self, self.end)
 
-        # Link this and the end using the relation 'Pairs'
-        Pairs(self, self.end)
+            # AsyncIO workers are used to be performed as computing threads
+            self.workers = []
 
-        # AsyncIO workers are used to be performed as computing threads
-        self.workers = []
-
-        # sim_network is a network containing all processes, actions, and items associated with simulation
-        self.sim_network = []
+            # sim_network is a network containing all processes, actions, and items associated with simulation
+            self.sim_network = []
 
     ########################################################################
     # Analysis Methods
@@ -1294,7 +1256,7 @@ class Process(DynamicEntity):
         """
         This checks interfaces between functions (or actions)
         """
-        from interfaceanalyzer import InterfaceAnalyzer as IA
+        from interface_analyzer import InterfaceAnalyzer as IA
         ia = IA(self)
         return ia.get_critical_elements(self), \
                ia.get_sphere_of_influence(self), \
@@ -1311,11 +1273,11 @@ class Process(DynamicEntity):
 
         entity_results, relation_results = self.search(class_search=[Requirement])
         for e in entity_results:
-            print(e)
+            # print(e)
             e.check_property()
 
     def get_action_times(self):
-        actions = Process.get_by_type(Action)
+        actions = entity_db.get_by_type(Action)
         dict_action = {x.name:x.total_time for x in actions}
         return dict_action
 
@@ -1370,13 +1332,6 @@ class Process(DynamicEntity):
             return None
         return Process.event_queue.get_nowait()
 
-    def save_simulation_results(self):
-        # self.print_flows(self.sim_network)
-
-        # 네트워크 서치로 프로퍼티와 관련 요구사항 리스트를 추출하는 것이 필요!!!
-        # print(self)
-        pass
-
     async def sim(self, until=5):
         """
         This triggers the simulation.
@@ -1386,18 +1341,6 @@ class Process(DynamicEntity):
         If until is None, the simulation performs just one time.
         """
 
-        # The function sim is called, which means this is a root process
-        self.is_root = True
-        self.end.is_root = True
-
-        # The root process is linked to 'process_END' using the relation 'flow'
-        # 'process_END' -> 'process'
-        self.end.flow(self)
-
-        # For the relation 'process_END' -> 'process', set 'sent = true' to activate the root process
-        flowed_from = [r for r in self.inv_relation['flowed from'] if isinstance(r, Flow)]
-        flowed_from[0].sent = True
-
         # Find other flows and set them into asyncio
         if 'flows' or 'contains' in self.relation:
             # Reset all global variables regarding simulation
@@ -1407,20 +1350,23 @@ class Process(DynamicEntity):
             Process.global_max = until
 
             # get flows from the relation 'flow' or 'contains'
-            self.init_sim_network()
+            new_en = self.init_sim_network()
 
             # print out control flows of UC
             # self.print_flows(self.sim_network)
 
-            workers = [x.run() for x in self.sim_network]
-            workers.append(self.run())
-            # await asyncio.gather(*self.workers)
+            workers = [x.run() for x in new_en.sim_network]
+            workers.append(new_en.run())
+
             try:
                 res = await asyncio.gather(*workers)
                 print('--------- Simulation Completed ---------')
+
+                del new_en
+
             except asyncio.CancelledError:
                 print('CancelledError')
-                res = None
+
 
     ########################################################################
     # Class Creation Methods
@@ -1466,14 +1412,22 @@ class Process(DynamicEntity):
 
         return obj
 
-    # def Process(self, name):
-    #     obj = Process(name)
-    #     Contains(self, obj)
-    #     Decomposes(self, obj)
-    #     return obj
+    def XOr(self, *args):
+        obj = XOr()
+        obj.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
 
-    def Loop(self, times=2):
-        obj = Loop(name='loop', times=times)
+        Contains(self, obj)
+
+        ret = []
+        if args is not None and len(args) > 0:
+            for proc in args:
+                ret.append(obj.Process(proc))
+            return tuple(ret)
+
+        return obj
+
+    def Loop(self, name, times=2):
+        obj = Loop(name=name, times=times)
         obj.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
 
         Contains(self, obj)
@@ -1519,3 +1473,48 @@ class Process(DynamicEntity):
 class Process_END(DynamicEntity):
     def __init__(self, name):
         super().__init__(name)
+
+
+class Loop(Process):
+    """
+    Loop Dynamic Entity
+    """
+
+    def __init__(self, name='loop', times=2):
+        super().__init__(name)
+        self.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
+
+        # This decomposes a pair dynamic entity called 'end' or name+'_END'
+        self.end = Loop_END(name + '_END', times=times)
+
+        # Link this and the end using the relation 'Pairs'
+        Pairs(self, self.end)
+
+        # 'loop' is linked to 'loop_END' using the relation 'flow'
+        # 'loop_END' -> 'loop'
+        self.end.flow(self)
+
+    ########################################################################
+    # Class Creation Methods
+    # !start with an uppercase letter
+    #
+    def Process(self, name):
+        obj = Process(name)
+        obj.is_root = False
+        obj.module, _ = os.path.splitext(traceback.extract_stack()[-2][0])
+
+        Contains(self, obj)
+        return obj
+
+
+class Loop_END(DynamicEntity):
+    def __init__(self, name, times=2):
+        super().__init__(name)
+
+        # 'loop_END' is set with the looping times
+        self.times = times
+        self.count = 0
+
+    def reset_waiting(self):
+        super().reset_waiting()
+        self.count = 0
